<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

if ( ! class_exists( 'EDD_Putler_Connector' ) ) {

	class EDD_Putler_Connector {

		private $name = 'EDD';

		public function __construct() {
			add_filter( 'putler_connector_get_order_count', array( &$this, 'get_order_count' ) );
			add_filter( 'putler_connector_get_orders', array( &$this, 'get_orders' ) );

			if ( is_admin() ) {
				add_action( 'edd_payment_delete', array( &$this, 'delete_post' ) );
				add_action( 'admin_init', array( &$this, 'delete_sub' ), 1 );
			}

		}

		//function to validate the transaction
		public function validate_transaction( $transaction ) {

			$response = array();

			$response_keys = array(
				'Date',
				'Time',
				'Time_Zone',
				'Source',
				'Name',
				'Type',
				'Status',
				'Currency',
				'Gross',
				'Fee',
				'Net',
				'From_Email_Address',
				'To_Email_Address',
				'Transaction_ID',
				'Counterparty_Status',
				'Address_Status',
				'Item_Title',
				'Item_ID',
				'Shipping_and_Handling_Amount',
				'Insurance_Amount',
				'Discount',
				'Sales_Tax',
				'Option_1_Name',
				'Option_1_Value',
				'Option_2_Name',
				'Option_2_Value',
				'Auction_Site',
				'Buyer_ID',
				'Item_URL',
				'Closing_Date',
				'Escrow_ID',
				'Invoice_ID',
				'Reference_Txn_ID',
				'Invoice_Number',
				'Custom_Number',
				'Quantity',
				'Receipt_ID',
				'Balance',
				'Note',
				'Address_Line_1',
				'Address_Line_2',
				'Town_City',
				'State_Province',
				'Zip_Postal_Code',
				'Country',
				'Contact_Phone_Number',
				'Subscription_ID',
				'Raw_Data',
				'Payment_Source',
				'External_Trans_ID',
				'IP_Address',
				'Sub_Meta'
			);


			$response_float_keys = array( 'Gross', 'Fee', 'Net', 'Shipping_and_Handling_Amount', 'Insurance_Amount', 'Discount', 'Sales_Tax', 'Balance' );

			foreach ( $response_keys as $key ) {

				$default_value = '';

				$float_value = in_array( $key, $response_float_keys );

				if ( ! empty( $float_value ) ) {
					$default_value = '0.00';
				} else {
					if ( $key == 'Currency' ) {
						$default_value = 'USD';
					} elseif ( $key == 'Sub_Meta' ) {
						$default_value = json_encode( json_decode( "{}" ) );
					}
				}

				$response[ $key ] = ( ! empty( $transaction[ $key ] ) ) ? $transaction[ $key ] : $default_value;
			}

			return $response;
		}

		//function to handle storing of the subscription transactions on deletion
		public function delete_sub() {

			if ( empty( $_POST['sub_id'] ) ) {
				return;
			}

			if ( empty( $_POST['edd_delete_subscription'] ) ) {
				return;
			}

			if ( ! current_user_can( 'edit_shop_payments' ) ) {
				return;
			}

			if ( ! wp_verify_nonce( $_POST['edd-recurring-update-nonce'], 'edd-recurring-update' ) ) {
				return;
			}

			global $wpdb;

			$sub_id        = absint( $_POST['sub_id'] );
			$modified_date = gmdate( 'Y-m-d H:i:s' );

			//array to store the transaction details
			$putler_transaction = array(
				'Date'            => gmdate( 'm/d/Y' ),
				'Time'            => gmdate( 'H:i:s' ),
				'Source'          => $this->name,
				'Time_Zone'       => 'GMT',
				'Type'            => 'Recurring Payment',
				'Status'          => 'Delete',
				'Transaction_ID'  => '',
				'Subscription_ID' => $sub_id
			);

			//query to fetch the sub details
			$query   = "SELECT parent_payment_id as order_id,
                            created as created_date,
                            notes
                        FROM {$wpdb->prefix}edd_subscriptions as edd_sub
                        WHERE id = " . $sub_id;
			$results = $wpdb->get_row( $query, 'ARRAY_A' );

			if ( count( $results ) > 0 ) {

				$notes_array = array_reverse( array_filter( explode( "\n\n", $results['notes'] ) ) );

				if ( ! empty( $notes_array ) ) {
					$gmt_offset_option = get_option( 'gmt_offset' );
					$gmt_offset        = ( ! empty( $gmt_offset_option ) ) ? $gmt_offset_option : 0;
					$modified_date     = substr( $notes_array[0], 0, strpos( $notes_array[0], ' - Status changed from' ) );
					$modified_date     = date( 'Y-m-d H:i:s', ( (int) strtotime( $modified_date ) ) - ( $gmt_offset * HOUR_IN_SECONDS ) );
				} else {
					$modified_date = date( "Y-m-d H:i:s", ( strtotime( $results['created_date'] ) + 300 ) );
				}

				$putler_transaction ['Transaction_ID'] = $results['order_id'] . '_D_' . strtotime( $modified_date );
			}

			if ( ! empty( $putler_transaction ) ) {
				set_transient( 'eddpc_deleted_' . time(), array( $putler_transaction ), ( DAY_IN_SECONDS * 180 ) );
			}
		}

		//function to handle storing of the transactions on deletion
		public function delete_post( $id ) {

			if ( empty( $id ) ) {
				return;
			}

			global $wpdb;

			$post_type = get_post_type( $id );

			$valid_post_type = array( "edd_payment" );

			if ( empty( $post_type ) || in_array( $post_type, $valid_post_type ) === false ) {
				return;
			}

			$post_status = get_post_status( $id );
			$post_parent = wp_get_post_parent_id( $id );


			//array to store the transaction details
			$putler_transaction = array(
				'Date'            => get_post_modified_time( 'm/d/Y', true, $id, false ),
				'Time'            => get_post_modified_time( 'H:i:s', true, $id, false ),
				'Source'          => $this->name,
				'Time_Zone'       => 'GMT',
				'Type'            => 'Shopping Cart Payment Received',
				'Status'          => 'Delete',
				'Transaction_ID'  => $id,
				'Subscription_ID' => ''
			);

			//query to fetch the subscriptions
			$query               = "SELECT edd_sub.id as sub_id,
                            edd_sub.product_id
                    FROM {$wpdb->prefix}edd_subscriptions as edd_sub
                        JOIN {$wpdb->prefix}posts as p
                            ON( (edd_sub.parent_payment_id = p.ID
                                OR (edd_sub.parent_payment_id = p.post_parent AND p.post_status = 'edd_subscription'))
                            AND p.post_type = 'edd_payment')
                    WHERE p.ID = " . $id;
			$results_sub_details = $wpdb->get_results( $query, 'ARRAY_A' );

			if ( count( $results_sub_details ) > 0 ) {

				if ( $putler_transaction['Type'] != 'Shopping Cart Payment Received' ) {
					$putler_transaction ['Subscription_ID'] = $results_sub_details[0]['sub_id'];
				} else {
					if ( count( $results_sub_details ) > 1 ) { //code for order having more than 1 subscription products in cart
						$putler_transaction['Type'] = 'Subscription Shopping Cart Payment Received';
					} else {

						$sub_p_ids = array();

						foreach ( $results_sub_details as $sub_detail ) {
							$sub_p_ids[ $sub_detail['product_id'] ] = $sub_detail['sub_id'];
						}

						//Query to get the Order_items
						$query_cart_items   = "SELECT postmeta.meta_value AS meta_value
                                                FROM {$wpdb->prefix}postmeta AS postmeta
                                                WHERE postmeta.meta_key = '_edd_payment_meta'
                                                        AND postmeta.post_id = " . $id;
						$results_cart_items = $wpdb->get_var( $query_cart_items );

						if ( ! empty( $results_cart_items ) ) {

							$sub = $prod = 0;

							$cart_items = maybe_unserialize( $results_cart_items );

							foreach ( $cart_items['cart_details'] as $cart_item ) {
								if ( ! empty( $sub_p_ids[ $cart_item['id'] ] ) ) {
									$sub ++;
								} else {
									$prod ++;
								}
							}

							//code for handling different subscription order conditions
							if ( $sub == 1 && $prod == 0 ) {
								$putler_transaction ['Type']            = 'Subscription Payment Received';
								$putler_transaction ['Subscription_ID'] = $results_sub_details[0]['sub_id'];
							} else if ( ( $sub > 1 && $prod == 0 ) || ( $sub >= 1 && $prod > 0 ) ) {
								$putler_transaction ['Type'] = 'Subscription Shopping Cart Payment Received';
							}
						}
					}
				}

			}

			if ( ! empty( $putler_transaction ) ) {
				set_transient( 'eddpc_deleted_' . time(), array( $putler_transaction ), ( DAY_IN_SECONDS * 180 ) );
			}

		}

		public function get_order_count( $count ) {
			global $wpdb;
			$order_count = 0;

			$query_to_fetch_order_count = "SELECT COUNT(posts.ID) as id
                                            FROM {$wpdb->prefix}posts AS posts
                                            JOIN {$wpdb->prefix}postmeta AS postmeta
                                            ON posts.ID = postmeta.post_id
                                            WHERE posts.post_type = 'edd_payment'
                                            AND postmeta.meta_key = '_edd_payment_mode'
                                            AND postmeta.meta_value = 'live'";

			$order_count_result = $wpdb->get_col( $query_to_fetch_order_count );
			if ( ! empty( $order_count_result ) ) {
				$order_count = $order_count_result[0];
			}

			return $count + $order_count;
		}

		public function new_status_map( $v ) {
			return $v[0];
		}

		//Function to get sub data recurring payment transactions
		public function get_sub_recurring_transaction( $params ) {

			if ( empty( $params ) || empty( $params['sub_id'] ) ) {
				return;
			}

			$status      = $params['sub_details']['status'];
			$status_abbr = implode( '', array_map( array( $this, 'new_status_map' ), explode( ' ', $status ) ) );

			$response = $transaction = array();

			$transaction ['Date']               = date( 'm/d/Y', (int) strtotime( $params['sub_details']['modified_date'] ) );
			$transaction ['Time']               = date( 'H:i:s', (int) strtotime( $params['sub_details']['modified_date'] ) );
			$transaction ['Time_Zone']          = 'GMT';
			$transaction ['Source']             = $this->name;
			$transaction ['Name']               = ( ! empty( $params['sub_details']['billing_name'] ) ) ? $params['sub_details']['billing_name'] : '';
			$transaction ['Type']               = 'Recurring Payment';
			$transaction ['Status']             = $status;
			$transaction ['Currency']           = ( ! empty( $params['sub_details']['order_currency'] ) ) ? $params['sub_details']['order_currency'] : 'USD';
			$transaction ['From_Email_Address'] = ( ! empty( $params['sub_details']['billing_email'] ) ) ? $params['sub_details']['billing_email'] : '';
			$transaction ['Transaction_ID']     = ( $status != 'Created' ) ? $params['sub_details']['parent_payment_id'] . '_' . $status_abbr . '_' . strtotime( $params['sub_details']['modified_date'] ) : $params['sub_id'];
			$transaction ['Subscription_ID']    = $params['sub_id'];

			$sub_details = array();

			foreach ( $params['sub_details'] as $key => $value ) {
				$sub_details[ $key ] = maybe_unserialize( $value );
			}

			$transaction ['Raw_Data'] = ( ! empty( $params['sub_details'] ) && $status == 'Created' ) ? json_encode( array( 'subscriptions' => array( $params['sub_id'] => $sub_details ) ) ) : '';
			$transaction ['Sub_Meta'] = json_encode( array(
				'payment_method' => $params['sub_details']['payment_method'],
				'interval'       => $params['sub_details']['interval'],
				'period'         => $params['sub_details']['period'],
				'trial_end'      => $params['sub_details']['trial_end'],
				'sub_end'        => $params['sub_details']['sub_end'],
				'status'         => $status
			) );

			$response = $this->validate_transaction( $transaction );

			return $response;
		}


		public function get_orders( $params ) {
			global $wpdb;

			$edd_order_status = array(
				'pending'          => 'Pending',
				'publish'          => 'Completed',
				'refunded'         => 'Refunded',
				'failed'           => 'Pending',
				'abandoned'        => 'Cancelled',
				'revoked'          => 'Cancelled',
				'edd_subscription' => 'Renewal',
				'trash'            => 'Delete'
			);

			$edd_sub_status = array(
				'pending'   => 'Pending',
				'active'    => 'Active',
				'cancelled' => 'Cancelled',
				'expired'   => 'Expired',
				'trialling' => 'Trialling',
				'failing'   => 'Failing',
				'completed' => 'Completed'
			);

			//Code to get the last order sent

			$cond = '';

			if ( empty( $params['order_id'] ) ) {
				// $start_limit = (isset($params[$this->name]['start_limit'])) ? $params[$this->name]['start_limit'] : 0;
				$start_limit = ( isset( $params['offset'] ) ) ? $params['offset'] : 0;
				$batch_limit = ( isset( $params['limit'] ) ) ? $params['limit'] : 50;
			} else {
				$start_limit = 0;
				$batch_limit = 1;
				$cond        = 'AND posts.ID IN(' . intval( $params['order_id'] ) . ')';
			}

			$query_order_details = "SELECT posts.ID as id,
                                            posts.post_status as order_status,
                                            date_format(posts.post_date_gmt,'%Y-%m-%d %T') AS date,
                                            date_format(posts.post_modified_gmt,'%Y-%m-%d %T') AS modified_date
                                    FROM {$wpdb->prefix}posts AS posts
                                        JOIN {$wpdb->prefix}postmeta AS postmeta
                                            ON posts.ID = postmeta.post_id
                                    WHERE posts.post_type LIKE 'edd_payment'
                                        AND posts.post_status NOT IN ('auto-draft', 'draft')
                                        AND posts.post_date_gmt != '0000-00-00 00:00:00'
                                        AND post_modified_gmt BETWEEN '" . $params['start_date'] . "' AND '" . $params['end_date'] . "'
                                        AND( (postmeta.meta_key = '_edd_payment_mode'
                                                AND postmeta.meta_value = 'live')
                                              OR (posts.post_parent > 0
                                                AND posts.post_status = 'edd_subscription'))
                                        AND posts.post_parent NOT IN(SELECT post_id FROM {$wpdb->prefix}postmeta WHERE meta_key = '_edd_payment_mode' AND meta_value = 'test')
                                        $cond
                                    GROUP BY posts.ID
                                    LIMIT " . $start_limit . "," . $batch_limit;

			$results_order_details       = $wpdb->get_results( $query_order_details, 'ARRAY_A' );
			$results_order_details_count = $wpdb->num_rows;

			if ( $results_order_details_count > 0 ) {

				$order_ids       = array();
				$order_trash_ids = array(); //to store the trashed order ids
				$sub_details = array();

				foreach ( $results_order_details as $results_order_detail ) {
					$order_ids[] = $results_order_detail['id'];

					//for trash order ids
					if ( ! empty( $results_order_detail['order_status'] ) && $results_order_detail['order_status'] == 'trash' ) {
						$order_trash_ids[] = $results_order_detail['id'];
					}
				}

				//Query to get the Order_items

				$query_cart_items   = "SELECT postmeta.post_id,
                                            postmeta.meta_key AS meta_key,
                                            postmeta.meta_value AS meta_value
                                    FROM {$wpdb->prefix}postmeta AS postmeta
                                    WHERE postmeta.meta_key NOT IN ('_edit_lock', '_edit_last')
                                            AND postmeta.post_id IN (" . implode( ",", $order_ids ) . ")
                                    GROUP BY postmeta.post_id, meta_key";
				$results_cart_items = $wpdb->get_results( $query_cart_items, 'ARRAY_A' );

				$results_cart_items_count = $wpdb->num_rows;

				$order_items = array();

				if ( $results_cart_items_count > 0 ) {

					foreach ( $results_cart_items as $cart_item ) {
						$order_id = $cart_item['post_id'];

						if ( ! isset( $order_items[ $order_id ] ) ) {
							$order_items[ $order_id ]                    = array();
							$order_items[ $order_id ]['tot_qty']         = 0;
							$order_items[ $order_id ]['payment_total']   = 0;
							$order_items[ $order_id ]['payment_details'] = array();
						}

						if ( $cart_item['meta_key'] == '_edd_payment_meta' ) {
							$order_meta_data = maybe_unserialize( $cart_item['meta_value'] );

							foreach ( $order_meta_data as $order_key => $order_data ) {
								$order_meta_data[ $order_key ] = is_serialized( $order_data ) ? maybe_unserialize( $order_data ) : $order_data;
							}

							$order_items[ $order_id ]['payment_details'] = $order_meta_data;
							$order_items[ $order_id ]['tot_qty']         = count( $order_items[ $order_id ]['payment_details']['cart_details'] );
						} else {
							$order_items[ $order_id ][ $cart_item['meta_key'] ] = $cart_item['meta_value'];
						}
					}
				}


				//Code for handling subscriptions meta data
				if ( in_array( 'edd-recurring/edd-recurring.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {

					//query to fetch the subscriptions
					$query               = "SELECT p.ID as order_id,
                                    edd_sub.id as sub_id,
                                    edd_sub.period,
                                    edd_sub.recurring_amount,
                                    edd_sub.parent_payment_id,
                                    edd_sub.product_id,
                                    edd_sub.created,
                                    edd_sub.expiration,
                                    edd_sub.trial_period,
                                    edd_sub.status,
                                    edd_sub.notes
                            FROM {$wpdb->prefix}edd_subscriptions as edd_sub
                                JOIN {$wpdb->prefix}posts as p
                                    ON( (edd_sub.parent_payment_id = p.ID
                                        OR (edd_sub.parent_payment_id = p.post_parent AND p.post_status = 'edd_subscription'))
                                    AND p.post_type = 'edd_payment')
                            WHERE p.ID IN (" . implode( ",", $order_ids ) . ")";
					$results_sub_details = $wpdb->get_results( $query, 'ARRAY_A' );

					if ( count( $results_sub_details ) > 0 ) {

						foreach ( $results_sub_details as $results_sub_detail ) {


							$order_id = $results_sub_detail['order_id'];
							$sub_id   = $results_sub_detail['sub_id'];

							if ( empty( $order_items[ $order_id ]['subscriptions'] ) ) {
								$order_items[ $order_id ]['subscriptions'] = array();
							}

							$order_items[ $order_id ]['subscriptions'][] = $results_sub_detail['sub_id'];

							if ( empty( $sub_details[ $sub_id ] ) ) {
								$sub_details[ $sub_id ] = array(
									'interval'          => 1,
									'period'            => $results_sub_detail['period'],
									'created_date'      => $results_sub_detail['created'],
									'trial_end'         => $results_sub_detail['trial_period'],
									'sub_end'           => $results_sub_detail['expiration'],
									'parent_payment_id' => $results_sub_detail['parent_payment_id'],
									'recurring_amount'  => $results_sub_detail['recurring_amount'],
									'product_id'        => $results_sub_detail['product_id'],
									'status'            => $edd_sub_status[ $results_sub_detail['status'] ],
									'notes'             => $results_sub_detail['notes'],
									'order_ids'         => array()
								);
							}

							$sub_details[ $sub_id ]['order_ids'][]    = $order_id;
							$sub_details[ $sub_id ]['payment_method'] = ( ! empty( $order_items[ $order_id ]['_edd_payment_gateway'] ) ) ? edd_get_gateway_admin_label( $order_items[ $order_id ]['_edd_payment_gateway'] ) : '';
							$sub_details[ $sub_id ]['billing_name']   = ( $order_items[ $order_id ]['payment_details']['user_info']['first_name'] . ' ' . $order_items[ $order_id ]['payment_details']['user_info']['last_name'] );
							$sub_details[ $sub_id ]['order_currency'] = $order_items[ $order_id ]['payment_details']['currency'];
							$sub_details[ $sub_id ]['billing_email']  = $order_items[ $order_id ]['payment_details']['user_info']['email'];

						}

						$gmt_offset_option = get_option( 'gmt_offset' );
						$gmt_offset        = ( ! empty( $gmt_offset_option ) ) ? $gmt_offset_option : 0;

						foreach ( $sub_details as $sub_id => $sub_detail ) { //for creating the recurring payment transactions

							if ( empty( $sub_detail['status'] ) ) { //code for ignoring the subscriptions with blank status
								continue;
							}

							$sub_details_og = $sub_detail;

							if ( trim( $sub_detail['status'] ) != 'Delete' ) { //for 'Created' Recurring transaction
								$sub_detail['status']        = 'Created';
								$sub_detail['modified_date'] = $sub_details[ $sub_id ]['created_date'];

								$args = array(
									'sub_id'      => $sub_id,
									'sub_details' => ( ( ! empty( $sub_detail ) ) ? $sub_detail : array() )
								);

								$sub_transactions[] = $this->get_sub_recurring_transaction( $args );
							}

							//Code for creating 'Recurring Payment' current status transaction for subscriptions
							$sub_detail = $sub_details_og;

							$modified_date = date( "Y-m-d H:i:s", ( strtotime( $sub_detail['created_date'] ) + 300 ) );

							$notes_array = array_reverse( array_filter( explode( "\n\n", $sub_detail['notes'] ) ) );

							if ( ! empty( $notes_array ) ) {

								foreach( $notes_array as $note ) {
									$pos1 = strpos( $note, ' - Status changed from' );									
									$pos2 = strpos( $note, 'to cancelled' );									

									if( $pos1 !== false && $pos2 !== false ) {
										$modified_date = substr( $note, 0, $pos1 );
										$modified_date = date( 'Y-m-d H:i:s', ( (int) strtotime( $modified_date ) ) - ( $gmt_offset * HOUR_IN_SECONDS ) );
										break;
									}
								}
							}

							$sub_detail['modified_date'] = $modified_date;
							$args                        = array(
								'sub_id'      => $sub_id,
								'sub_details' => ( ( ! empty( $sub_detail ) ) ? $sub_detail : array() )
							);

							$sub_transactions[] = $this->get_sub_recurring_transaction( $args );
						}
					}
				}


				if ( $results_order_details > 0 ) {

					//Code for Data Mapping as per Putler
					foreach ( $results_order_details as $order_detail ) {

						$sub_trans = false;

						$order_id     = $order_detail['id'];
						$date_gmt     = $order_detail['date'];
						$dateInGMT    = date( 'm/d/Y', (int) strtotime( $date_gmt ) );
						$timeInGMT    = date( 'H:i:s', (int) strtotime( $date_gmt ) );
						$order_status = $order_detail['order_status'];
						$order_total  = $order_items[ $order_id ]['_edd_payment_total'];

						if ( empty( $order_status ) ) { //code for ignoring the orders with blank status
							continue;
						}

						// $response['date_time'] = $date_gmt;
						$response ['Date']      = $dateInGMT;
						$response ['Time']      = $timeInGMT;
						$response ['Time_Zone'] = 'GMT';

						$response ['Source'] = $this->name;
						$response ['Name']   = $order_items[ $order_id ]['payment_details']['user_info']['first_name'] . ' ' . $order_items[ $order_id ]['payment_details']['user_info']['last_name'];
						// $response ['Type'] = ( $status == "refunded") ? 'Refund' : 'Shopping Cart Payment Received';
						$response ['Type']   = 'Shopping Cart Payment Received';
						$response ['Status'] = ( ! empty( $edd_order_status[ $order_status ] ) && $edd_order_status[ $order_status ] == 'Renewal' ) ? 'Completed' : $edd_order_status[ $order_status ];

						$response ['Currency'] = $order_items[ $order_id ]['payment_details']['currency'];

						$response ['Gross'] = $order_total;
						$response ['Fee']   = 0.00;
						$response ['Net']   = $order_total;

						$response ['From_Email_Address']           = $order_items[ $order_id ]['payment_details']['user_info']['email'];
						$response ['To_Email_Address']             = '';
						$response ['Transaction_ID']               = $order_id;
						$response ['Counterparty_Status']          = '';
						$response ['Address_Status']               = '';
						$response ['Item_Title']                   = 'Shopping Cart';
						$response ['Item_ID']                      = 0; // Set to 0 for main Order Transaction row
						$response ['Shipping_and_Handling_Amount'] = 0.00;
						$response ['Insurance_Amount']             = '';
						$response ['Discount']                     = 0.00;

						$response ['Sales_Tax'] = ( ! empty( $order_items[ $order_id ]['_edd_payment_tax'] ) ) ? round( $order_items[ $order_id ]['_edd_payment_tax'], 2) : 0.00;

						$response ['Option_1_Name']  = '';
						$response ['Option_1_Value'] = '';
						$response ['Option_2_Name']  = '';
						$response ['Option_2_Value'] = '';

						$response ['Auction_Site']     = '';
						$response ['Buyer_ID']         = '';
						$response ['Item_URL']         = '';
						$response ['Closing_Date']     = '';
						$response ['Escrow_ID']        = '';
						$response ['Invoice_ID']       = '';
						$response ['Reference_Txn_ID'] = '';
						$response ['Invoice_Number']   = '';
						$response ['Custom_Number']    = '';
						$response ['Quantity']         = $order_items[ $order_id ]['tot_qty'];
						$response ['Receipt_ID']       = '';

						$response ['Balance']              = '';
						$response ['Note']                 = ''; // No customer notes.
						$response ['Address_Line_1']       = ( isset( $order_items[ $order_id ]['payment_details']['user_info']['address']['line1'] ) ) ? $order_items[ $order_id ]['payment_details']['user_info']['address']['line1'] : '';
						$response ['Address_Line_2']       = isset( $order_items[ $order_id ]['payment_details']['user_info']['address']['line2'] ) ? $order_items[ $order_id ]['payment_details']['user_info']['address']['line2'] : '';
						$response ['Town_City']            = isset( $order_items[ $order_id ]['user_info']['payment_details']['address']['city'] ) ? $order_items[ $order_id ]['payment_details']['user_info']['address']['city'] : '';
						$response ['State_Province']       = isset( $order_items[ $order_id ]['payment_details']['user_info']['address']['state'] ) ? $order_items[ $order_id ]['payment_details']['user_info']['address']['state'] : '';
						$response ['Zip_Postal_Code']      = isset( $order_items[ $order_id ]['payment_details']['user_info']['address']['zip'] ) ? $order_items[ $order_id ]['payment_details']['user_info']['address']['zip'] : '';
						$response ['Country']              = isset( $order_items[ $order_id ]['payment_details']['user_info']['address']['country'] ) ? $order_items[ $order_id ]['payment_details']['user_info']['address']['country'] : '';
						$response ['Contact_Phone_Number'] = ''; // TODO
						$response ['Subscription_ID']      = '';


						//code for handling subscriptions
						if ( ! empty( $order_items[ $order_id ]['subscriptions'] ) ) {

							$sub = $prod = 0;

							foreach ( $order_items[ $order_id ]['payment_details']['cart_details'] as $cart_item ) {

								$is_sub = false;

								foreach ( $sub_details as $sub_id => $sub_detail ) {

									$sub_order_ids = ( ! empty( $sub_detail['order_ids'] ) ) ? $sub_detail['order_ids'] : array();
									$valid_sub     = in_array( $order_id, $sub_order_ids );

									if ( $cart_item['id'] == $sub_detail['product_id'] && ! empty( $valid_sub ) ) {
										$is_sub = true;
										break;
									}
								}

								if ( ! empty( $is_sub ) ) {
									$sub ++;
								} else {
									$prod ++;
								}
							}

							//code for handling different subscription order conditions
							if ( $sub == 1 && $prod == 0 ) {
								$response ['Type']            = 'Subscription Payment Received';
								$response ['Subscription_ID'] = ( ! empty( $order_items[ $order_id ]['subscriptions'] ) ) ? $order_items[ $order_id ]['subscriptions'][0] : '';
								$sub_trans                    = true;
							} else if ( ( $sub > 1 && $prod == 0 ) || ( $sub >= 1 && $prod > 0 ) ) {
								$response ['Type'] = 'Subscription Shopping Cart Payment Received';
							}
						}

						if ( $order_detail['order_status'] != 'trash' ) {

							if ( ! empty( $order_items[ $order_id ]['subscriptions'] ) ) {

								$sub_ids = $order_items[ $order_id ]['subscriptions'];

								$order_items[ $order_id ]['subscriptions'] = array();

								foreach ( $sub_ids as $sub_id ) {
									$order_items[ $order_id ]['subscriptions'][ $sub_id ] = ( ! empty( $sub_details[ $sub_id ] ) ) ? $sub_details[ $sub_id ] : array();
								}
							}

							$order_meta = array();
							foreach ( $order_items[ $order_id ] as $key => $value ) {
								$order_meta[ $key ] = maybe_unserialize( $value );
							}

							$response ['Raw_Data'] = json_encode( $order_meta );
						} else {
							$response ['Raw_Data'] = '';
						}

						// if ($order_status != "refunded") {
						$response ['Payment_Source']    = ( ! empty( $order_items[ $order_id ]['_edd_payment_gateway'] ) ) ? edd_get_gateway_admin_label( $order_items[ $order_id ]['_edd_payment_gateway'] ) : '';
						$response ['External_Trans_ID'] = ( ! empty( $order_items[ $order_id ]['_edd_payment_transaction_id'] ) ) ? $order_items[ $order_id ]['_edd_payment_transaction_id'] : '';
						// }

						//customer ip_address
						$response ['IP_Address'] = ( ! empty( $order_items[ $order_id ]['_edd_payment_user_ip'] ) ) ? $order_items[ $order_id ]['_edd_payment_user_ip'] : '';

						$response ['Sub_Meta'] = json_encode( json_decode( "{}" ) ); //for subscription meta

						if ( empty( $sub_trans ) ) {
							$transactions [] = $response;
						}


						foreach ( $order_items[ $order_id ]['payment_details']['cart_details'] as $cart_item ) {

							$order_item                = array();
							$order_item ['Type']       = ( $sub_trans === false && $response ['Type'] != 'Refund' ) ? 'Shopping Cart Item' : $response ['Type'];
							$order_item ['Item_Title'] = $cart_item['name'];
							$order_item ['Item_ID']    = $cart_item['id'];
							$order_item ['Gross']      = round( $cart_item['price'], 2 );
							$order_item ['Quantity']   = $cart_item['quantity'];

							if ( isset( $cart_item['item_number']['options']['price_id'] ) ) {
								// Do code for option name and option value

								$price_id                     = $cart_item['item_number']['options']['price_id'];
								$order_item['Option_1_Name']  = '';
								$order_item['Option_1_Value'] = edd_get_price_option_name( $cart_item['id'], $price_id, $order_id );
							}

							if ( !empty( $cart_item['item_number']['options']['recurring'] ) ) { //code for adding the sign up fee in case of subscriptions
								if ( !empty( $cart_item['item_number']['options']['recurring']['signup_fee'] ) ) {
									$order_item ['Gross'] = round( ($order_item ['Gross'] + $cart_item['item_number']['options']['recurring']['signup_fee']), 2 );
								}
							}

							//for subscription meta data
							foreach ( $sub_details as $sub_id => $sub_detail ) {

								$sub_order_ids = ( ! empty( $sub_detail['order_ids'] ) ) ? $sub_detail['order_ids'] : array();
								$valid_sub     = in_array( $order_id, $sub_order_ids );

								if ( $cart_item['id'] == $sub_detail['product_id'] && ! empty( $valid_sub ) ) {
									$order_item ['Subscription_ID'] = ( ! empty( $sub_id ) ) ? $sub_id : ''; //Assigning the subscription id

									if ( $order_detail['order_status'] != 'trash' ) {
										$response ['Sub_Meta'] = array(
											'payment_method' => $sub_detail['payment_method'],
											'interval'       => $sub_detail['interval'],
											'period'         => $sub_detail['period'],
											'trial_end'      => $sub_detail['trial_end'],
											'sub_end'        => $sub_detail['sub_end'],
											'status'         => $sub_detail['status']
										);

										if ( isset( $sub_detail['parent_payment_id'] ) && $order_id != $sub_detail['parent_payment_id'] ) {
											$response ['Sub_Meta']['is_renewal'] = 1;
										}

										$response ['Sub_Meta'] = json_encode( $response ['Sub_Meta'] );

									}

									break;
								}
							}

							if ( $order_detail['order_status'] != 'trash' ) {
								$response ['Raw_Data'] = ( $sub_trans === false || $response ['Type'] == 'Refund' ) ? '' : $response ['Raw_Data'];
							} else {
								$response ['Raw_Data'] = '';
							}

							$transactions [] = array_merge( $response, $order_item );

							if ( $order_status == "refunded" ) {

								$date_gmt_modified = $order_detail['modified_date'];

								$response ['Date'] = date( 'm/d/Y', (int) strtotime( $date_gmt_modified ) );
								$response ['Time'] = date( 'H:i:s', (int) strtotime( $date_gmt_modified ) );

								$response ['Type']             = 'Refund';
								$response ['Status']           = 'Completed';
								$response ['Gross']            = - $order_total;
								$response ['Net']              = - $order_total;
								$response ['Transaction_ID']   = $order_id . '_R';
								$response ['Reference_Txn_ID'] = $order_id;

								$response ['Payment_Source'] = $response ['External_Trans_ID'] = ''; //for type refund transactions the same should be blank

								$transactions [] = $response;
							}
						}
					}
				}

				if ( empty( $params['order_id'] ) ) {

					//Code for fetching the deleted transactions
					$query = "SELECT option_value 
                                FROM {$wpdb->prefix}options
                                WHERE option_name LIKE '%_transient_eddpc_deleted_%' 
                                    AND convert_tz(FROM_UNIXTIME(SUBSTRING_INDEX(option_name,'_transient_eddpc_deleted_',-1)),@@session.time_zone,'+00:00') BETWEEN '" . $params['start_date'] . "' AND '" . $params['end_date'] . "'";

					$results = $wpdb->get_col( $query );

					if ( count( $results ) > 0 ) {
						foreach ( $results as $result ) {

							$deleted_trans = maybe_unserialize( $result );

							foreach ( $deleted_trans as $trans ) {
								$transactions[] = $this->validate_transaction( $trans );
							}
						}
					}

					$order_count           = ( is_array( $results_order_details ) ) ? count( $results_order_details ) : 0;
					$data                  = ( ! empty( $transactions ) ) ? $transactions : array();
					$data                  = ( ! empty( $sub_transactions ) ) ? array_merge( $data, $sub_transactions ) : $data;
					$params[ $this->name ] = array( 'count' => $order_count, 'last_start_limit' => $start_limit, 'data' => $data );
				} else {
					$params['data'] = $transactions;
				}
			} else {

			}

			return $params;
		}

	}

}
