<?php

/*
 * Plugin Name: 		Easy Digital Downloads Putler Connector
 * Plugin URI: 			http://putler.com/connector/edd/
 * Description: 		Track Easy Digital Downloads transactions data with Putler. Insightful reporting that grows your business.
 * Version: 			2.5.8.1
 * Requires at least: 	3.3
 * Tested up to: 		5.3.2
 * Author: 				putler, storeapps
 * Author URI: 			http://putler.com/
 * License:           	GPL 3.0
 * Copyright (c)      	2016, 2017, 2018, 2019, 2020 putler, All right reserved
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

if( !defined('EPC_VERSION') ) {
	define( 'EPC_VERSION', '2.5.8.1' );
}


//Hooks
register_activation_hook( __FILE__, 'eddpc_activate' );
add_action( 'plugins_loaded', 'eddpc_load' );


/**
 * Registers a plugin function to be run when the plugin is activated.
 */
function eddpc_activate() {
	// Redirect to eddpc
	update_option( '_eddpc_activation_redirect', 'pending' );
}

$eddpc_notice_msg = '';

function eddpc_load() {

	global $eddpc_notice_msg;

	$eddpc_text_domain = 'putler_connector';

	$active_plugins = (array) get_option( 'active_plugins', array() );

	if ( is_multisite() ) {
		$active_plugins = array_merge( $active_plugins, get_site_option( 'active_sitewide_plugins', array() ) );
	}

	add_action( 'admin_notices', 'eddpc_admin_notices' );

	if ( ( ! in_array( 'woocommerce-putler-connector/woocommerce-putler-connector.php', $active_plugins ) && ! array_key_exists( 'woocommerce-putler-connector/woocommerce-putler-connector.php', $active_plugins ) )
	     && ( ! in_array( 'jigoshop-putler-connector/jigoshop-putler-connector.php', $active_plugins ) && ! array_key_exists( 'jigoshop-putler-connector/jigoshop-putler-connector.php', $active_plugins ) )
	     && ( ! in_array( 'wp-e-commerce-putler-connector/wpec-putler-connector.php', $active_plugins ) && ! array_key_exists( 'wp-e-commerce-putler-connector/wpec-putler-connector.php', $active_plugins ) ) ) {

		$eddpc_notice_msg = '';

		if ( in_array( 'easy-digital-downloads/easy-digital-downloads.php', $active_plugins ) || array_key_exists( 'easy-digital-downloads/easy-digital-downloads.php', $active_plugins ) ) {

			if ( ! defined( 'PUTLER_GATEWAY' ) ) {
				define( 'PUTLER_GATEWAY', 'EDD' );
			}

			if ( ! defined( 'PUTLER_GATEWAY_PREFIX' ) ) {
				define( 'PUTLER_GATEWAY_PREFIX', 'eddpc' );
			}

			include_once 'classes/class.putler-connector.php';
			$GLOBALS['putler_connector'] = Putler_Connector::getInstance();
			include_once 'classes/class.putler-connector-edd.php';
			if ( ! isset( $GLOBALS['edd_putler_connector'] ) ) {
				$GLOBALS['edd_putler_connector'] = new EDD_Putler_Connector();
			}

			add_action( 'admin_init', 'eddpc_init' );
		} else {

			$eddpc_notice_msg = '<div id="notice" class="error"><p>' .
			                    '<b>' . __( 'Easy Digital Downloads Putler Connector', $eddpc_text_domain ) . '</b> ' . __( 'add-on requires', $eddpc_text_domain ) . ' <a href="https://wordpress.org/plugins/easy-digital-downloads/">' . __( 'Easy Digital Downloads', $eddpc_text_domain ) . '</a> ' . __( 'plugin. Please install and activate it.', $eddpc_text_domain ) .
			                    '</p></div>';
		}

	} else {
		$eddpc_notice_msg = '<div id="notice" class="error"><p>' .
		                    __( 'Any one of the Putler Connector\'s can be active at any given time. Please <b>deactivate all the other Putler Connector\'s.</b>', $eddpc_text_domain ) . '</b> ' .
		                    '</p></div>';
	}
}

function eddpc_admin_notices() {

	global $eddpc_notice_msg;

	if ( ! empty( $eddpc_notice_msg ) ) {
		echo $eddpc_notice_msg;
	}

}


function eddpc_init() {
	// Init admin menu for settings etc if we are in admin
	if ( is_admin() ) {

		if ( false === get_option( '_eddpc_update_redirect_257' ) && 'pending' !== get_option( '_eddpc_activation_redirect' ) ) {
			update_option( '_eddpc_update_redirect_257', 1 ); //flag for redirecting on update
			update_option( '_eddpc_delete_and_resync', 1 ); //flag for delete and resyncing the data on update
			delete_option( '_eddpc_update_redirect' );
			update_option( '_eddpc_activation_redirect', 'pending' );
		}

		if ( false !== get_option( '_eddpc_activation_redirect' ) && ( current_user_can( 'import' ) === true ) ) {
			// Delete the redirect transient
			delete_option( '_eddpc_activation_redirect' );
			wp_redirect( admin_url( 'tools.php?page=putler_connector&action=eddpc_activate' ) );
			exit;
		}

	}
}
